#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu May  2 17:56:49 2019

Class for handling data saving.

For now, this is how it works: you choose a directory and you create a Storage object with the directory as input. 
Then, the method create_data_file creates a data file in dump, to which append_data_to_current_file can be used to load data 
into it. That lies in the dump, and can me made permanent with permanent_save_current_data. 

@author: martindrech
"""

import numpy as np
import datetime
import os
import shutil

class Storage(object):
    """
    Class to handle data saving. 
    """
    def __init__(self, directory):
        
        self.directory = directory
        
    def create_data_file(self, *args):
        saving_directory = "/".join([self.directory, self.get_date_string()])
        if not os.path.exists(saving_directory):
            os.makedirs(saving_directory)
            os.makedirs(saving_directory + '/dump')

        s_num = self.get_saving_number()
       
        filename = "/".join([saving_directory, 'dump', 's' + str(s_num).zfill(2)]) + '.temp'
        header = " ".join([s for s in args]+['\n'])

        with open(filename, 'wb') as f:
            f.write(header.encode('utf-8'))
        
        
    def append_data_to_current_file(self, *args):
        with open(self.get_current_filename(), 'ab') as f:
            data = np.column_stack(args)
            np.savetxt(f, data)
            f.flush()

    def get_date_string(self):
        y = datetime.datetime.now().year
        m = datetime.datetime.now().month
        d = datetime.datetime.now().day
        s = str(y) + str(m).zfill(2) + str(d).zfill(2)
        return s
        
    def get_saving_number(self, permanent_saving = False):
        
        savings_number_list = []
        if permanent_saving:
            suffix = '.dat'
            saving_directory = "/".join([self.directory, self.get_date_string()])
        else:
            suffix = '.temp'
            saving_directory = "/".join([self.directory, self.get_date_string(), 'dump'])
        
        for s in [dat_file for dat_file in os.listdir(saving_directory) if dat_file.endswith(suffix)]:
            savings_number_list.append(int(s[1:].split('.')[0]))
        
        if len(savings_number_list) == 0:
            return 1      
        if len(savings_number_list) == 3 and not permanent_saving: #removes the last of the last 10 files
            file_to_remove = "/".join([saving_directory, 's' + str(min(savings_number_list)).zfill(2)])+'.temp'
            os.remove(file_to_remove)            
            
        return max(savings_number_list)+1
    
    def get_current_filename(self):
        s_num = self.get_saving_number()-1
        saving_directory = "/".join([self.directory, self.get_date_string()])
        filename = "/".join([saving_directory, 'dump', 's' + str(s_num).zfill(2)]) + '.temp'
        return filename
    
    
    def permanent_save_current_data(self):
        oldfile = self.get_current_filename()
        saving_directory = "/".join([self.directory, self.get_date_string()])
        newfilename = 's' + str(self.get_saving_number(permanent_saving = True) ).zfill(2) + '.dat'
        shutil.move(oldfile, "/".join([saving_directory, newfilename]) )
        print("/".join([saving_directory, newfilename]))

#%%

if __name__ == '__main__':
    storage = Storage('/home/martindrech/Documents/storage')
    storage.create_data_file('col1', 'col2')
